local addonName = ...

local function useIcon(icon)
	if not (MacroFrame:IsShown() and IsControlKeyDown()) then
		return
	elseif InCombatLockdown() then
		return true
	end
	icon = icon:match("([^\\]+)$")
	EditMacro(MacroFrame.selectedMacro, nil, icon)
	MacroFrame_Update()
	if MacroPopupFrame:IsShown() then
		local mode = MacroPopupFrame.mode
		MacroPopupFrame.mode = nil
		MacroPopupFrame_Update(MacroPopupFrame)
		MacroPopupFrame.mode = mode
	end
	return true
end


local spellButton_OnModifiedClick = SpellButton_OnModifiedClick

function SpellButton_OnModifiedClick(self, button)
	if useIcon(GetSpellBookItemTexture(SpellBook_GetSpellBookSlot(self), SpellBookFrame.bookType)) then
		return
	end
	
	spellButton_OnModifiedClick(self, button) 
end

-- items
local handleModifiedItemClick = HandleModifiedItemClick

function HandleModifiedItemClick(link)
	if not link then
		return
	end

	if useIcon(GetItemIcon(link)) then
		return true
	end
	
	return handleModifiedItemClick(link)
end

local addons = {
	Blizzard_AchievementUI = function()
		local achievementButton_OnClick = AchievementButton_OnClick

		function AchievementButton_OnClick(self, button, down, ignoreModifiers)
			if useIcon(select(10, GetAchievementInfo(self.id))) then
				return
			end
			
			achievementButton_OnClick(self, button, down, ignoreModifiers)
		end
		
		local achievementFrameSummaryAchievement_OnClick = AchievementFrameSummaryAchievement_OnClick
		
		function AchievementFrameSummaryAchievement_OnClick(self)
			if useIcon(select(10, GetAchievementInfo(self.id))) then
				return
			end
			
			achievementFrameSummaryAchievement_OnClick(self)
		end
	end,
	Blizzard_PetJournal = function()
		do
			local buttons = MountJournal.ListScrollFrame.buttons
			local old_onClick = buttons[1].DragButton:GetScript("OnClick")
			local function onClick(self, button)
				if useIcon(select(4, GetCompanionInfo("MOUNT", self:GetParent().index))) then
					return
				end
				old_onClick(self, button)
			end
			
			for i, button in ipairs(buttons) do
				button.DragButton:SetScript("OnClick", onClick)
			end
		end
		
		do
			local buttons = PetJournal.listScroll.buttons
			local old_onClick = buttons[1].dragButton:GetScript("OnClick")
			local function onClick(self, button)
				if useIcon(select(9, C_PetJournal.GetPetInfoByIndex(self:GetParent().index, PetJournal.isWild))) then
					return
				end
				old_onClick(self, button)
			end
			
			for i, button in ipairs(buttons) do
				button.dragButton:SetScript("OnClick", onClick)
			end
		end
	end,
	Blizzard_TalentUI = function()
		local playerTalentFrameTalent_OnClick = PlayerTalentFrameTalent_OnClick

		function PlayerTalentFrameTalent_OnClick(self, button)
			if useIcon(select(2, GetTalentInfo(self:GetID(), nil, PlayerTalentFrameTalents.talentGroup))) then
				return
			end

			playerTalentFrameTalent_OnClick(self, button)
		end
	end,
}


local frame = CreateFrame("Frame")
frame:RegisterEvent("ADDON_LOADED")
frame:SetScript("OnEvent", function(self, event, ...)
	self[event](self, ...)
end)

function frame:ADDON_LOADED(addon)
	if addon == addonName then
		for addon, loader in pairs(addons) do
			if IsAddOnLoaded(addon) then
				loader()
				addons[addon] = nil
			end
		end
	else
		if addons[addon] then
			addons[addon]()
			addons[addon] = nil
		end
	end
	
	if not next(addons) then
		self:UnregisterEvent("ADDON_LOADED")
	end
end